# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.0.0

import winrt.system
from winrt._winrt_windows_media_faceanalysis import (
    DetectedFace,
    FaceDetector,
    FaceTracker,
)

__all__ = [
    "DetectedFace",
    "FaceDetector",
    "FaceTracker",
]

